@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2016 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2018 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -32,11 +32,6 @@
 #include <stb_image.h>
 #define STB_IMAGE_WRITE_IMPLEMENTATION
 #include <stb_image_write.h>
-extern "C"
-{
-    #include <jpeglib.h>
-    #include <jerror.h>
-}
 #include <cctype>
 
 
@@ -268,7 +263,7 @@ bool ImageLoader::saveImageToFile(const std::string& filename, const std::vector
         else if (extension == "jpg" || extension == "jpeg")
         {
             // JPG format
-            if (writeJpg(filename, pixels, size.x, size.y))
+            if (stbi_write_jpg(filename.c_str(), size.x, size.y, 4, &pixels[0], 90))
                 return true;
         }
     }
@@ -277,60 +272,6 @@ bool ImageLoader::saveImageToFile(const std::string& filename, const std::vector
     return false;
 }
 
-
-////////////////////////////////////////////////////////////
-bool ImageLoader::writeJpg(const std::string& filename, const std::vector<Uint8>& pixels, unsigned int width, unsigned int height)
-{
-    // Open the file to write in
-    FILE* file = fopen(filename.c_str(), "wb");
-    if (!file)
-        return false;
-
-    // Initialize the error handler
-    jpeg_compress_struct compressInfos;
-    jpeg_error_mgr errorManager;
-    compressInfos.err = jpeg_std_error(&errorManager);
-
-    // Initialize all the writing and compression infos
-    jpeg_create_compress(&compressInfos);
-    compressInfos.image_width      = width;
-    compressInfos.image_height     = height;
-    compressInfos.input_components = 3;
-    compressInfos.in_color_space   = JCS_RGB;
-    jpeg_stdio_dest(&compressInfos, file);
-    jpeg_set_defaults(&compressInfos);
-    jpeg_set_quality(&compressInfos, 90, TRUE);
-
-    // Get rid of the alpha channel
-    std::vector<Uint8> buffer(width * height * 3);
-    for (std::size_t i = 0; i < width * height; ++i)
-    {
-        buffer[i * 3 + 0] = pixels[i * 4 + 0];
-        buffer[i * 3 + 1] = pixels[i * 4 + 1];
-        buffer[i * 3 + 2] = pixels[i * 4 + 2];
-    }
-    Uint8* ptr = &buffer[0];
-
-    // Start compression
-    jpeg_start_compress(&compressInfos, TRUE);
-
-    // Write each row of the image
-    while (compressInfos.next_scanline < compressInfos.image_height)
-    {
-        JSAMPROW rawPointer = ptr + (compressInfos.next_scanline * width * 3);
-        jpeg_write_scanlines(&compressInfos, &rawPointer, 1);
-    }
-
-    // Finish compression
-    jpeg_finish_compress(&compressInfos);
-    jpeg_destroy_compress(&compressInfos);
-
-    // Close the file
-    fclose(file);
-
-    return true;
-}
-
 } // namespace priv
 
 } // namespace sf
