@@ -178,6 +178,20 @@ void WindowImplAndroid::setMouseCursorVisible(bool visible)
 }
 
 
+////////////////////////////////////////////////////////////
+void WindowImplAndroid::setMouseCursorGrabbed(bool grabbed)
+{
+    // Not applicable
+}
+
+
+////////////////////////////////////////////////////////////
+void WindowImplAndroid::setMouseCursor(const CursorImpl& cursor)
+{
+    // Not applicable
+}
+
+
 ////////////////////////////////////////////////////////////
 void WindowImplAndroid::setKeyRepeatEnabled(bool enabled)
 {
@@ -404,7 +418,7 @@ int WindowImplAndroid::processKeyEvent(AInputEvent* _event, ActivityStates* stat
         if (key == AKEYCODE_UNKNOWN)
         {
             // This is a unique sequence, which is not yet exposed in the NDK
-            // http://code.google.com/p/android/issues/detail?id=33998
+            // https://code.google.com/p/android/issues/detail?id=33998
             return 0;
         }
         else if (int unicode = getUnicode(_event)) // This is a repeated sequence
@@ -433,7 +447,7 @@ int WindowImplAndroid::processMotionEvent(AInputEvent* _event, ActivityStates* s
 
     if (device == AINPUT_SOURCE_MOUSE)
         event.type = Event::MouseMoved;
-    else if (device == AINPUT_SOURCE_TOUCHSCREEN)
+    else if (device & AINPUT_SOURCE_TOUCHSCREEN)
         event.type = Event::TouchMoved;
 
     int pointerCount = AMotionEvent_getPointerCount(_event);
@@ -452,7 +466,7 @@ int WindowImplAndroid::processMotionEvent(AInputEvent* _event, ActivityStates* s
 
             states->mousePosition = Vector2i(event.mouseMove.x, event.mouseMove.y);
         }
-        else if (device == AINPUT_SOURCE_TOUCHSCREEN)
+        else if (device & AINPUT_SOURCE_TOUCHSCREEN)
         {
             if (states->touchEvents[id].x == x && states->touchEvents[id].y == y)
                 continue;
@@ -496,7 +510,7 @@ int WindowImplAndroid::processPointerEvent(bool isDown, AInputEvent* _event, Act
             if (id >= 0 && id < Mouse::ButtonCount)
                 states->isButtonPressed[id] = true;
         }
-        else if (device == AINPUT_SOURCE_TOUCHSCREEN)
+        else if (device & AINPUT_SOURCE_TOUCHSCREEN)
         {
             event.type = Event::TouchBegan;
             event.touch.finger = id;
@@ -518,7 +532,7 @@ int WindowImplAndroid::processPointerEvent(bool isDown, AInputEvent* _event, Act
             if (id >= 0 && id < Mouse::ButtonCount)
                 states->isButtonPressed[id] = false;
         }
-        else if (device == AINPUT_SOURCE_TOUCHSCREEN)
+        else if (device & AINPUT_SOURCE_TOUCHSCREEN)
         {
             event.type = Event::TouchEnded;
             event.touch.finger = id;
@@ -605,15 +619,16 @@ Keyboard::Key WindowImplAndroid::androidKeyToSF(int32_t key)
         case AKEYCODE_SYM:
         case AKEYCODE_EXPLORER:
         case AKEYCODE_ENVELOPE:           return Keyboard::Unknown;
-        case AKEYCODE_ENTER:              return Keyboard::Return;
-        case AKEYCODE_DEL:                return Keyboard::Delete;
+        case AKEYCODE_ENTER:              return Keyboard::Enter;
+        case AKEYCODE_DEL:                return Keyboard::Backspace;
+        case AKEYCODE_FORWARD_DEL:        return Keyboard::Delete;
         case AKEYCODE_GRAVE:              return Keyboard::Tilde;
         case AKEYCODE_MINUS:              return Keyboard::Subtract;
         case AKEYCODE_EQUALS:             return Keyboard::Equal;
         case AKEYCODE_LEFT_BRACKET:       return Keyboard::LBracket;
         case AKEYCODE_RIGHT_BRACKET:      return Keyboard::RBracket;
-        case AKEYCODE_BACKSLASH:          return Keyboard::BackSlash;
-        case AKEYCODE_SEMICOLON:          return Keyboard::SemiColon;
+        case AKEYCODE_BACKSLASH:          return Keyboard::Backslash;
+        case AKEYCODE_SEMICOLON:          return Keyboard::Semicolon;
         case AKEYCODE_APOSTROPHE:         return Keyboard::Quote;
         case AKEYCODE_SLASH:              return Keyboard::Slash;
         case AKEYCODE_AT:
