@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2017 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2018 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -80,6 +80,13 @@ const Context* Context::getActiveContext()
 }
 
 
+////////////////////////////////////////////////////////////
+Uint64 Context::getActiveContextId()
+{
+    return priv::GlContext::getActiveContextId();
+}
+
+
 ////////////////////////////////////////////////////////////
 bool Context::isExtensionAvailable(const char* name)
 {
