@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2016 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2018 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -90,6 +90,10 @@ struct JoystickState
 
     #include <SFML/Window/FreeBSD/JoystickImpl.hpp>
 
+#elif defined(SFML_SYSTEM_OPENBSD)
+
+    #include <SFML/Window/OpenBSD/JoystickImpl.hpp>
+
 #elif defined(SFML_SYSTEM_MACOS)
 
     #include <SFML/Window/OSX/JoystickImpl.hpp>
