@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2017 Marco Antognini (antognini.marco@gmail.com),
+// Copyright (C) 2007-2018 Marco Antognini (antognini.marco@gmail.com),
 //                         Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
@@ -136,7 +136,7 @@
     if ([NSThread currentThread] != [NSThread mainThread])
     {
         /*
-         * See http://lists.apple.com/archives/cocoa-dev/2011/Feb/msg00460.html
+         * See https://lists.apple.com/archives/cocoa-dev/2011/Feb/msg00460.html
          * for more information.
          */
         sf::err() << "Cannot create a window from a worker thread. (OS X limitation)" << std::endl;
@@ -371,6 +371,13 @@
 }
 
 
+////////////////////////////////////////////////////////
+-(void)setCursor:(NSCursor*)cursor
+{
+    return [m_oglView setCursor:cursor];
+}
+
+
 ////////////////////////////////////////////////////////////
 -(NSPoint)position
 {
@@ -559,7 +566,7 @@
     if ([NSThread currentThread] != [NSThread mainThread])
     {
         /*
-         * See http://lists.apple.com/archives/cocoa-dev/2011/Feb/msg00460.html
+         * See https://lists.apple.com/archives/cocoa-dev/2011/Feb/msg00460.html
          * for more information.
          */
         sf::err() << "Cannot fetch event from a worker thread. (OS X restriction)" << std::endl;
