@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2016 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2018 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -31,6 +31,8 @@
 #include <SFML/Config.hpp>
 #include <SFML/System/NonCopyable.hpp>
 #include <SFML/System/String.hpp>
+#include <SFML/Window/ContextSettings.hpp>
+#include <SFML/Window/CursorImpl.hpp>
 #include <SFML/Window/Event.hpp>
 #include <SFML/Window/Joystick.hpp>
 #include <SFML/Window/JoystickImpl.hpp>
@@ -38,7 +40,7 @@
 #include <SFML/Window/SensorImpl.hpp>
 #include <SFML/Window/VideoMode.hpp>
 #include <SFML/Window/WindowHandle.hpp>
-#include <SFML/Window/ContextSettings.hpp>
+#include <SFML/Window/Window.hpp>
 #include <queue>
 #include <set>
 
@@ -194,6 +196,14 @@ public:
     ////////////////////////////////////////////////////////////
     virtual void setMouseCursorGrabbed(bool grabbed) = 0;
 
+    ////////////////////////////////////////////////////////////
+    /// \brief Set the displayed cursor to a native system cursor
+    ///
+    /// \param cursor Native system cursor type to display
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual void setMouseCursor(const CursorImpl& cursor) = 0;
+
     ////////////////////////////////////////////////////////////
     /// \brief Enable or disable automatic key-repeat
     ///
@@ -260,10 +270,11 @@ private:
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    std::queue<Event> m_events;                          ///< Queue of available events
-    JoystickState     m_joystickStates[Joystick::Count]; ///< Previous state of the joysticks
-    Vector3f          m_sensorValue[Sensor::Count];      ///< Previous value of the sensors
-    float             m_joystickThreshold;               ///< Joystick threshold (minimum motion for "move" event to be generated)
+    std::queue<Event> m_events;                                              ///< Queue of available events
+    JoystickState     m_joystickStates[Joystick::Count];                     ///< Previous state of the joysticks
+    Vector3f          m_sensorValue[Sensor::Count];                          ///< Previous value of the sensors
+    float             m_joystickThreshold;                                   ///< Joystick threshold (minimum motion for "move" event to be generated)
+    float             m_previousAxes[Joystick::Count][Joystick::AxisCount];  ///< Position of each axis last time a move event triggered, in range [-100, 100]
 };
 
 } // namespace priv
